@@ -16,10 +16,11 @@
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include <iostream>
-#include <string>
-#include <vector>
 
+#include <config.h>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+
+#include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/input/eclipse/Units/Units.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 #include <opm/input/eclipse/Deck/DeckRecord.hpp>
@@ -27,11 +28,15 @@
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 
-#include "injection.hpp"
 #include "../eval_uda.hpp"
 
+#include <fmt/format.h>
+
+#include <ostream>
+#include <string>
+#include <vector>
+
 namespace Opm {
 
 
@@ -47,7 +52,6 @@ namespace Opm {
           reservoirInjectionRate(units.getDimension(UnitSystem::measure::rate)),
           BHPTarget(units.getDimension(UnitSystem::measure::pressure)),
           THPTarget(units.getDimension(UnitSystem::measure::pressure)),
-          temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
           BHPH(0),
           THPH(0),
           VFPTableNumber(0),
@@ -69,7 +73,6 @@ namespace Opm {
         result.THPTarget = UDAValue(3.0);
         result.bhp_hist_limit = 4.0;
         result.thp_hist_limit = 5.0;
-        result.temperature = 6.0;
         result.BHPH = 7.0;
         result.THPH = 8.0;
         result.VFPTableNumber = 9;
@@ -124,7 +127,7 @@ namespace Opm {
             this->dropInjectionControl(InjectorCMode::GRUP);
         {
             const std::string& cmodeString = record.getItem("CMODE").getTrimmedString(0);
-            InjectorCMode controlModeArg = InjectorCModeFromString( cmodeString );
+            InjectorCMode controlModeArg = WellInjectorCModeFromString(cmodeString);
             if (this->hasInjectionControl( controlModeArg))
                 this->controlMode = controlModeArg;
             else {
@@ -177,7 +180,12 @@ namespace Opm {
     }
 
 
-    void Well::WellInjectionProperties::handleWCONINJH(const DeckRecord& record, bool is_producer, const std::string& well_name) {
+    void
+    Well::WellInjectionProperties::handleWCONINJH(const DeckRecord& record,
+                                                  const bool is_producer,
+                                                  const std::string& well_name,
+                                                  const KeywordLocation& loc)
+    {
         // convert injection rates to SI
         const auto& typeItem = record.getItem("TYPE");
         if (typeItem.defaultApplied(0)) {
@@ -196,11 +204,22 @@ namespace Opm {
             this->THPH = record.getItem("THP").getSIDouble(0);
 
         const std::string& cmodeString = record.getItem("CMODE").getTrimmedString(0);
-        const InjectorCMode newControlMode = InjectorCModeFromString( cmodeString );
+        InjectorCMode newControlMode = WellInjectorCModeFromString(cmodeString);
 
         if ( !(newControlMode == InjectorCMode::RATE || newControlMode == InjectorCMode::BHP) ) {
-            const std::string msg = "Only RATE and BHP control are allowed for WCONINJH for well " + well_name;
-            throw std::invalid_argument(msg);
+            newControlMode = InjectorCMode::RATE;
+            const auto& sir = this->surfaceInjectionRate;
+            std::string target = sir.is<double>() ? std::to_string(sir.get<double>()) : sir.get<std::string>();
+            std::string msg = fmt::format("Problem with keyword WCONINJH\n"
+                                          "In {} line {}\n"
+                                          "Only RATE and BHP controls supported for well {}.\n"
+                                          "Selected control {} reset to RATE, with target = {}.",
+                                          loc.filename,
+                                          loc.lineno,
+                                          well_name,
+                                          WellInjectorCMode2String(newControlMode),
+                                          target);
+            OpmLog::warning(msg);
         }
 
         // when well is under BHP control, we use its historical BHP value as BHP limit
@@ -238,7 +257,6 @@ namespace Opm {
     bool Well::WellInjectionProperties::operator==(const Well::WellInjectionProperties& other) const {
         if ((surfaceInjectionRate == other.surfaceInjectionRate) &&
             (reservoirInjectionRate == other.reservoirInjectionRate) &&
-            (temperature == other.temperature) &&
             (BHPTarget == other.BHPTarget) &&
             (THPTarget == other.THPTarget) &&
             (BHPH == other.BHPH) &&
@@ -280,7 +298,6 @@ namespace Opm {
             << "Well::WellInjectionProperties { "
             << "surfacerate: "      << wp.surfaceInjectionRate << ", "
             << "reservoir rate "    << wp.reservoirInjectionRate << ", "
-            << "temperature: "      << wp.temperature << ", "
             << "BHP target: "       << wp.BHPTarget << ", "
             << "THP target: "       << wp.THPTarget << ", "
             << "BHPH: "             << wp.BHPH << ", "
@@ -289,7 +306,7 @@ namespace Opm {
             << "prediction mode: "  << wp.predictionMode << ", "
             << "injection ctrl: "   << wp.injectionControls << ", "
             << "injector type: "    << InjectorType2String(wp.injectorType) << ", "
-            << "control mode: "     << Well::InjectorCMode2String(wp.controlMode) << " , "
+            << "control mode: "     << WellInjectorCMode2String(wp.controlMode) << " , "
             << "rs/rv concentration: " << wp.rsRvInj << " }";
     }
 
@@ -306,7 +323,6 @@ namespace Opm {
             controls.bhp_limit = this->bhp_hist_limit;
             controls.thp_limit = this->thp_hist_limit;
         }
-        controls.temperature = this->temperature;
         controls.injector_type = this->injectorType;
         controls.cmode = this->controlMode;
         controls.vfp_table_number = this->VFPTableNumber;
